from django_webid.provider.models import CertConfig
from django_webid.provider import provider_settings as app_settings

#site-wide django settings conf is parsed
#in webidprovider_settings, and there a sensible default is provided.

def set_defaultsettings(sender, **kwargs):

    dc = CertConfig()
    dc_dict = app_settings.CertDefaultsWrapper()
    for field, value in dc_dict.items():
        setattr(dc, field, value)
    dc.save()



from django.db import models

from django.contrib.auth.models import User
from django.contrib.sites.models import Site

from django.core.urlresolvers import reverse
#from django.utils.translation import gettext as _


class SingletonManager(models.Manager):
    def single(self):
        return self.all()[0]


class CertConfig(models.Model):
    """
    this model stores several defaults used
    in the certificate creation.
    values can be changed by site admin.
    On syncdb it is populated from settings, falling
    back to a certain set of defaults.
    This model behaves as a singleton: its admin
    does not allow adding or deleting the single instance.
    """

    #Defaults for App Behavior
    hide_keygen_form = models.BooleanField(default=False,
    help_text="WebID Certificate Add Page will hide keygen \
details from user, giving the impression of an automatic certificate \
installation")

    #Defaults for subject

    #XXX should be choices from ISO-2 country codes.
    country_name = models.CharField(null=True, blank=True, max_length=2,
            help_text="Country Name (2 letter code)")
    state = models.CharField(null=True, blank=True, max_length=255,
            help_text="State or Province")
    locality = models.CharField(null=True, blank=True, max_length=255)
    organization = models.CharField(null=True, blank=True, max_length=255,
            help_text="Organization Name")
    organizational_unit = models.CharField(null=True, blank=True,
            max_length=255,
            help_text="Organizational Unit Name")

    #XXX TBD.. We can mark here a gfk to contenttypes
    #so site admin can decide which field goes
    #into the common name.
    #RIGHT NOW XXX THIS IS QUITE PRONE TO MISTAKES :(
    common_name_field = models.CharField(default="nick", max_length=255,
        help_text="User Field to be\
        used in the Certificate's Subject Common Name")

    #XXX Actually in the future,
    #it would make sense to eval all the other fields
    #(admin can decide to map the value of the certificate on some fields to
    #user-related fields
    #in the database.

    #Defaults for validity
    valid_for_days = models.IntegerField(help_text="How many days will the\
        certificate be valid")
    valid_from_days = models.IntegerField(help_text="How many days, starting\
        with the cert creation, until the certificate is active")

    def get_subject_data(self):
        """
        helper methods for certificate creation.
        returns a dict with cert subject codes
        and field values
        """
        data = {}
        attrs = (('C', 'country_name'),
                 ('ST', 'state'),
                 ('L', 'locality'),
                 ('O', 'organization'),
                 ('OU', 'organizational_unit'))
        for code, field_name in attrs:
            val = getattr(self, field_name, None)
            if val:
                data[code] = val
        return data

    def get_common_name_field(self):
        return self.common_name_field

    def get_validity(self):
        data = {"for_days": self.valid_for_days,
                "from_days": self.valid_from_days}
        return data

    #currently using a hack: see Admin no-add/no-edit restriction
    #we can decide whether it makes sense having a per-site config section
    #or not...

    objects = SingletonManager()

    class Meta:
        verbose_name = "Default Certificate Values"
        verbose_name_plural = "Default Certificate Values"
        db_table = "webid_provider_certconfig"

    def __unicode__(self):
        return "Default Configuration for WebID Certificates"


class ActivePubKeyManager(models.Manager):
    """
    default manager for PubKey. returns only
    the PubKeys with is_active = True
    """
    def get_query_set(self):
        return super(ActivePubKeyManager,
                self).get_query_set().filter(is_active=True)


class PubKey(models.Model):
    #XXX in admin, should use inline
    #for viewing the cert at the same time.

    mod = models.CharField(max_length=10000)
    exp = models.IntegerField()
    bits = models.IntegerField()

    #choices?
    pkey_algorithm = models.CharField(blank=True, max_length=255)
    is_active = models.BooleanField(default=True)

    created = models.DateTimeField(blank=True)

    user = models.ForeignKey(User)

    #we preserve the original manager
    objects = models.Manager()
    #we only count the active PubKeys
    active_objects = ActivePubKeyManager()

    @property
    def date_created(self):
        return self.created.strftime('%m/%d/%Y')

    def __unicode__(self):
        if not hasattr(self, 'cert'):
            return "%s %s" % (
                self.user.username,
                self.date_created)
        else:
            return "%s %s * %s" % (
                self.user,
                self.date_created,
                self.cert.fingerprint_sha1)

    class Meta:
        verbose_name = "Public Key"
        verbose_name_plural = "Public Keys"
        db_table = "webid_provider_pubkey"


class ActiveCertManager(models.Manager):
    """
    default manager for Certs. returns only
    the Certs with PubKeys in which is_active = True
    """
    def get_query_set(self):
        return super(ActiveCertManager,
                self).get_query_set().filter(pubkey__is_active=True)


class Cert(models.Model):
    """
    Class for storing certificate info.
    """
    #XXX we can also store here info from visiting Users.
    #So we check certstr on auth.
    #created_by_us = False

    pubkey = models.OneToOneField(PubKey)

    valid_from = models.DateTimeField(blank=True)
    expires = models.DateTimeField(blank=True)

    #XXX property: days until expiration.
    #a task can be used to warn users about
    #upcoming expirations.

    #fingerprint is a property of PubKey or of Cert???
    fingerprint_sha256 = models.CharField(max_length=95)
    fingerprint_sha1 = models.CharField(max_length=60)
    fingerprint_md5 = models.CharField(max_length=50)

    #It can help the user to recognize
    #in which browser was installed...
    user_agent_string = models.CharField(max_length=255, null=True, blank=True)
    comment = models.CharField(max_length=255, null=True, blank=True)

    #algorithm??
    #Format (pkcs10, 12, 7...)

    objects = ActiveCertManager()
    all_objects = models.Manager()

    class Meta:
        verbose_name = "x509 Certificate"
        verbose_name_plural = "x509 Certificates"
        #app_label = "django_webid_provider"
        db_table = "webid_provider_cert"

    def __unicode__(self):
        return "%s's cert with Fingerprint %s" % (
                self.pubkey.user.username,
                self.fingerprint_sha1)


class WebIDURI(models.Model):
    """
    This class stores the URI for WebIDUsers.
    (The URI where we retrieved the WebIDProfile for
    external users; the URI that we have assigned
    thru a callback on hosted users.
    It can be accessed through WebIDUser proxy model.
    """
    uri = models.URLField(null=True, blank=True)
    user = models.ForeignKey(User, unique=True,
            related_name="stored_webiduris")
    #External/Internal (hosted) webid users.
    _is_user_hosted_here = models.BooleanField()
    #@property internal_user / external_user
    #We can still add some fields (here?)
    #related to the de-referenced uri
    #i.e., timestamp of profile fetching, content-type, ...
    #that can help to the periodical re-fetching.

    class Meta:
        #app_label = "django_webid_provider"
        db_table = "webid_provider_webiduri"


class WebIDUser(User):
    """
    Proxy model for accessing WebIDUsers.
    Implements methods for filtering own/external users, and
    retrieving the absolute WebID URI.
    """

    class Meta:
        proxy = True

    #XXX have to think about this.
    #but here we can
    # - add managers (to separate "hosted" users from external users)
    # - add properties (shortcuts to attr / methods on the profile)

    @staticmethod
    def get_for_user(user):
        return WebIDUser.objects.get(id=user.id)

    @staticmethod
    def get_for_uri(uri):
        try:
            return WebIDUser.objects.get(stored_webiduris__uri=uri)
        except WebIDUser.DoesNotExist:
            return None

    def _get_webid_url(self):
        #XXX FIXME
        #if it is hosted (see checking profile)
        #return reverse url
        #XXX in the future, we can use any other
        #user field. Use contenttypes in CertConfig.
        return reverse('webidprovider-webid_uri',
                kwargs={'username': self.username})

    webid_url = property(_get_webid_url)
    #XXX in the property, need to hook a configurable callback.
    #XXX should raise exception on _set,
    #or use proper fields.

    def _get_absolute_webid_uri(self):
        #XXX should do something about the HTTP(S)
        #for the future

        #Hmm what about putting https and then http?
        #Does the spec handle the recursion gracefully?
        #webid.auth module does not.

        #XXX document that we NEED TO HAVE SITES APP WORKING.
        #XXX get a setting for this?

        uri = "http://%s%s#me" % (
                Site.objects.get_current().domain,
                self._get_webid_url())
        return uri

    absolute_webid_uri = property(_get_absolute_webid_uri)

    @property
    def keys(self):
        #only active keys (keys have overriden manager)
        return self.pubkey_set.all()

    def _get_uri(self):
        """
        property that retrieves the webiduri object
        via FK.
        """
        #XXX we MUST hook here a possible callback to retrieve uris.
        uris = self.stored_webiduris.all()
        if uris.count() > 0:
            return uris[0]
    #XXX this is confusing with the webid_url and absoulte_webid_uri above!!!
    storeduri = property(_get_uri)

    def __unicode__(self):
        return self.username


class WebIDProfile(models.Model):
    """
    Abstract model for different WebID profiles.
    Optional Use (fallback)
    """
    #XXX we could try to use
    #zope.interface here... (so to enforce all child models
    #have a certain set of methods).
    #we can also define a stub for each method??.

    #we store here the webid_uri for external users.
    #for internal users you should use the absolute_webid_uri
    #property of WebIDUser

    #XXX Move this to WebIDUser ^^^
    #uri = models.URLField()
    user = models.OneToOneField(WebIDUser)

    #XXX move this to WebIDUser ^^^

    class Meta:
        abstract = True
        #app_label = "django_webid_provider"


class WebIDBasicProfile(WebIDProfile):
    """"
    Sample implementation of WebIDProfile
    """
    first_name = models.CharField(max_length=255)
    last_name = models.CharField(max_length=255)

    class Meta:
        #app_label = "django_webid_provider"
        db_table = "webid_provider_webidbasicprofile"



from django.conf import settings

#THIS IS THE DEFAULTS DICT FOR CERTCONFIG and app-wide
#behaviors.
#DO NOT change them here: use the site-wide settings file
#or change them in the CertConfig Admin interface.

SENSIBLE_DEFAULTS = {
        #subject name
        "CERT_C": ("country_name", "CC"),
        "CERT_ST": ("state", "Cyberspace"),
        "CERT_L": ("locality", "/dev/null"),
        "CERT_O": ("organization", "FOAF+SSL"),
        "CERT_OU": ("organizational_unit", "The Community of Self Signers"),
        #name field
        "CERT_CN" : ("common_name_field", "username"),
        #validity
        "CERT_VALID_FOR_DAYS": ("valid_for_days", 365),
        "CERT_VALID_FROM_DAYS": ("valid_from_days", 0)
}

APP_NAME = "WEBID_PROVIDER"

#XXX TODO
#Document in sphinx docs all the possible default settings WEBID_PROVIDER_XXX

class CertDefaultsWrapper(dict):
    def __init__(self):
        for cert_field, (model_field, default) in SENSIBLE_DEFAULTS.items():
            self[model_field] = getattr(settings,
            "%s_%s" % (APP_NAME, cert_field),
            default)





from django.conf.urls.defaults import *
from django.views.generic.simple import redirect_to
from django.views.generic.list_detail import object_list
from django_webid.provider import views, webiduri
from django_webid.provider.models import Cert

urlpatterns = patterns('',
    #Main, WORKING urls

    #temporary REDIRECT
    # XXX move to the sample project
    url(r'^$', redirect_to, {'url':'cert/add'}),
    url(r'^logout$', views.logout_view, name="webidprovider-logout"),

    ###################################
    #BEGIN django_webid.provider views

    #certs/all/ --> take all objects from pubkey manager (currently should
    #be only active)
    url(r'^certs$', views.cert_list_by_user, name="webidprovider-cert_list"),
    url(r'^cert/add$', views.add_cert_to_user, name="webidprovider-add_cert"),
    url(r'^cert/(?P<cert_id>\d+)/$', views.cert_detail,
        name="webidprovider-cert-detail"),
    url(r'^cert/(?P<cert_id>\d+)/revoke$', views.cert_revoke,
        name="webidprovider-cert-revoke"),

    #Our simple user creation view.
    #XXX we should move it to example site too.
    url(r'^user/add$', views.create_user, name='webidprovider-create_user'),

    #WebID Profile / Foaf publishing...
    #XXX can we do some magic here? (asking settings, finstance)
    #XXX should be "%s/foo" % settings.WEBID_PATH
    #XXX We MUST allow for other app to take control of this
    #XXX but at the same time provide a fallback mechanism...

    url(r'^(?P<username>\w+)$',
        webiduri.WebIDProfileView.as_view(), name="webidprovider-webid_uri"),

    ################################################
    ################################################
    # Some other, older tests: to be cleaned from here...
    # Might be BROKEN

    url(r'^cert/p12/add$', views.webid_identity,
        name='webidprovider-webid_identity1'),
    url(r'^cert/keygen$', views.webid_identity_keygen,
        name='webidprovider-webid_identity_keygen'),

)



from django import forms
from django.utils.safestring import mark_safe

class CharNotEditableWidget(forms.Widget):
    def render(self, name, value, attrs):
        final_attrs = self.build_attrs(attrs, name=name)
        if hasattr(self, 'initial'):
            value = self.initial
        return mark_safe(
            "@%s" % (
                value
            )
        )
    def _has_changed(self, initial, data):
        return False

class WebIdIdentityForm(forms.Form):
    nick = forms.CharField(max_length=10)
    webid = forms.URLField(required=False)



import base64
import logging
import re
from datetime import datetime

from OpenSSL import crypto
import M2Crypto as m2crypto

#XXX GOD KILLS A KITTEN EVERY TIME
#WE USE BOTH WRAPPERS TOGETHER!!!
#Hopefully, Santa bring us an improved m2crypto API
#since we've been good people...


class CertificateCreationError(Exception):
    pass


class CertOptions(object):
    pass


class CertCreator(object):
    """
    The magic class that hides
    all the x509 operations,
    _and_ saves PubKey and Cert
    instances to the database.
    """

    def __init__(self, spkac_str, user, **kwargs):
        """
        Receive spkac_str and create cert.
        On the **kwargs dict we can receive custom
        parameters for the cert creation (from advanced views).
        """
        #XXX FIXME get spkac_str as a kw arg.
        self.user_agent_string = kwargs.get('user_agent_string', None)
        self.skip_sign = kwargs.get('skip_sign', False)

        self._get_cert_configs(**kwargs)
        self._get_user_data(user)
        self._get_csr(spkac_str)
        self._verify_csr()
        self._init_cert()

    #####################################################
    # Private Methods
    # (mostly sequential CRYPTO FUNCTIONS)
    #####################################################

    def _get_cert_configs(self, **kwargs):
        from django_webid.provider.models import CertConfig
        certconfig = CertConfig.objects.single()

        #get datadicts from certconfig
        default_subject_data = certconfig.get_subject_data()
        default_cn_field = certconfig.get_common_name_field()
        default_validity_data = certconfig.get_validity()

        opt = CertOptions()
        opt.subject_data = default_subject_data
        opt.cn_field = default_cn_field
        opt.validity_data = default_validity_data

        #Now we override with **kwargs fields
        #coming from advanced options view form.
        #XXX get OVERRIDES values from the defaults settings!
        overrides = {}
        POSSIBLE_OVERRIDES = (
                "from_days",
                "for_days",
                "valid_from_ts",
                "expires_ts",
                )
        for k in POSSIBLE_OVERRIDES:
            if kwargs.has_key(k):
                overrides[k] = kwargs[k]

        #validity overrides
        for k in opt.validity_data.keys():
            if k in overrides.keys():
                opt.validity_data[k] = overrides[k]

        for k in ('valid_from_ts', 'expires_ts'):
            if k in overrides.keys():
                opt.validity_data[k] = overrides[k]

        #XXX TODO: FINISH OVERRIDES
        #XXX TODO: we can flatten the options
        #with a small namespace: val_data_foobar
        #might split...

        #issuerKey override (for test purposes mainly)
        issuerKey = kwargs.get('issuerKey',None)
        if issuerKey:
            self.issuerKey = issuerKey

        self.opts = opt


    def _get_user_data(self, user):
        from django_webid.provider.models import WebIDUser
        u = WebIDUser.get_for_user(user)
        webid = u.absolute_webid_uri

        #XXX check for errors
        self.URI_STR = 'URI:%s' % (webid)
        #to be used in subjectAltName
        self.webiduser = u

    def _get_csr(self, spkac_str):
        """
        gets a NetscapeSPKI object from the spkac_str
        or loads a PKCS10
        """
        # TODO should figure out how to use this
        # challenge thing (spkac is using md5, so
        # I guess there should be some time bound)
        # maybe linked to session or something???
        # See https://www.w3.org/Bugs/Public/show_bug.cgi?id=13518
        #http://www.w3.org/wiki/Foaf%2Bssl/Clients#Keygen
        # We could der-decode the spkac and get the challenge from there.

        spkac_str = re.sub('\s', '', spkac_str)
        #print('SPKAC PUBKEY=%s' % spkac_str)
        logging.debug('SPKAC PUBKEY=%s' % spkac_str)

        if "BEGINIECERTIFICATEREQUEST" in spkac_str:
            #XXX improve csr type detection (get a flag from view)
            pkcs10 = spkac_str.replace(
                    '-----BEGINIECERTIFICATEREQUEST-----','').replace(
                    '-----ENDIECERTIFICATEREQUEST-----','')

            der = base64.b64decode(pkcs10)
            self.csr = crypto.load_certificate_request(
                    crypto.FILETYPE_ASN1, der)
            self.csr_type = "msie"

        else:
            self.csr = crypto.NetscapeSPKI(spkac_str)
            self.csr_type = "spki"

    def _verify_csr(self):
        pk = self.csr.get_pubkey()
        check = self.csr.verify(pk)
        if check != 1L:
            raise CertificateCreationError('Server could not verify the CSR')

    def _init_cert(self):
        """
        create an empty cert
        """
        self.cert = crypto.X509()


    # ***
    # begins crypto functions that alter the certificate

    def _create_key_pair(self, _type, bits):
        pkey = crypto.PKey()
        pkey.generate_key(_type, bits)
        return pkey

    def _set_cert_subject(self):
        for code, val in self.opts.subject_data.items():
            setattr(self.cert.get_subject(), code, val)
        #XXX in the UID we can also write the webid URI

    def _create_issuer(self):
        icert = crypto.X509()
        #XXX get this from CertConfig ?
        #XXX have it cached somewhere?
        issuer_data = (('CN', 'Not a Certification Authority'),
                        ('O', 'FOAF+SSL'),
                        ('OU', 'The Community of Self Signers'))
        for k,v in issuer_data:
            setattr(icert.get_subject(), k, v)
        self.icert = icert

    def _set_issuer(self):
        self._create_issuer()
        self.cert.set_issuer(self.icert.get_subject())

    def _set_common_name(self):
        cn_field = getattr(self.webiduser,
                self.opts.cn_field,
                None)
        if not cn_field:
            cn_field = gettattr(self.webiduser,
                    'username')
        self.cert.get_subject().CN = cn_field

    def _set_pubkey(self):
        """
        gets the pubkey from the certificate request
        and adds it to the x509 cert.
        """
        pk = self.csr.get_pubkey()
        self.cert.set_pubkey(pk)

    def _set_fake_serial(self):
        """
        we replace this serial later,
        but seems we need a well-formed cert
        to be able to dump it... not needed
        after we get over the dirty workaround
        between M2 and PyOpenSSL...
        """
        self.cert.set_serial_number(0)

    def _set_validity(self):
        """
        sets validity parameters
        """
        val_data = self.opts.validity_data
        fmt = '%Y%m%d%H%M%SZ'
        #XXX Dont trust this data!
        #if not int, asssume 0 / 365

        nbts = val_data.get('valid_from_ts', None)
        if nbts and isinstance(nbts, datetime):
            self.cert.set_notBefore(nbts.strftime(fmt))

        else:
            not_before = 0 + (24 * 60 * 60 *
                    val_data['from_days'])
            self.cert.gmtime_adj_notBefore(not_before)

        nats = val_data.get('expires_ts', None)
        if nats and isinstance(nats, datetime):
            self.cert.set_notAfter(nats.strftime(fmt))
        else:
            #XXX We should allow also
            #advanced validity data in hours.
            valid_until = (24 * 60 * 60 *
                    val_data['for_days'])
            self.cert.gmtime_adj_notAfter(valid_until)

        # get the final cert validity data for
        # saving it in the db
        vdata = {}
        cnb = self.cert.get_notBefore()
        cna = self.cert.get_notAfter()
        vdata['notBefore'] = datetime.strptime(cnb, fmt)
        vdata['notAfter'] = datetime.strptime(cna, fmt)
        self.vdata_cert = vdata

    def _set_subject_alt_name(self):
        """
        here comes the magic for WebID!
        """
        ext = crypto.X509Extension('subjectAltName',
                1,
                self.URI_STR)
        self.cert.add_extensions([ext])

    def _set_version(self):
        # version 3 (decimal)
        self.cert.set_version(2)

    def _sign(self, issuerKey, digest="sha1"):
        self.cert.sign(issuerKey, digest)

    def _get_openssl_mod_exp(self):
        import binascii
        #@self.cert is an instance of OpenSSL.crypto.X509
        #dirty workaround with M2Crypto
        #FIXME!!!
        dump = crypto.dump_certificate(crypto.FILETYPE_PEM,
                self.cert)
        m2c = m2crypto.X509.load_cert_string(dump)
        pubkey = m2c.get_pubkey()
        mod = pubkey.get_modulus().lower()

        pkrsa = pubkey.get_rsa()
        e  = m2crypto.m2.rsa_get_e(pkrsa.rsa)
        ha = binascii.hexlify(e)
        #not correct
        #but it's not likely >3
        nbytes = ha.lstrip('0')[0]
        exp = int(ha[-int(nbytes)*2:],16)
        bits = pubkey.size() * 8

        return (mod, exp, bits)


    ##############################
    # done with crypto functions
    # Now: writing model instances
    ##############################

    #XXX can we decouple the instance writting
    #from the crypto functions??
    #--- As two separate classes...

    def _set_cert_serial_number(self, serial):
        self.cert.set_serial_number(serial)

    def _save_pubkey_instance(self):
        from django_webid.provider.models import PubKey

        mod, exp, bits = self._get_openssl_mod_exp()

        now = datetime.now()
        pk_instance = PubKey.objects.create(mod=mod,
                exp=exp,
                bits=bits,
                created=now,
                user=self.webiduser)

        #XXX TODO get Public Key Algorithm
        #from cert
        #and write into db

        #XXX could change this. We can have a table only
        #for serial counter, this not accurate.
        #But using the id of PubKey instances
        #is a good enough approximation.
        self._set_cert_serial_number(pk_instance.id)
        self.pk_instance = pk_instance

    def _set_cert_dump(self):
        if self.csr_type == "msie":
            filetype = crypto.FILETYPE_PEM
        else:
            filetype = crypto.FILETYPE_ASN1
        self.cert_dump = crypto.dump_certificate(
            filetype,
            self.cert)


    def _get_cert_hashes(self):
        """
        returns sha1 and md5 hashes of the
        certificate dump, as colon-delimited
        strings.
        """
        import hashlib
        beautify = lambda s: ':'.join([s[i:i+2] \
                for i in xrange(0, len(s), 2)])
        return (beautify(
                    hashlib.sha1(self.cert_dump).\
                            hexdigest()),
                beautify(
                    hashlib.md5(self.cert_dump).\
                            hexdigest()),
                beautify(
                    hashlib.sha256(self.cert_dump).\
                            hexdigest()))

    def _save_cert_instance(self):
        from django_webid.provider.models import Cert
        c = Cert()
        c.fingerprint_sha1, \
        c.fingerprint_md5,\
        c.fingerprint_sha256 = self._get_cert_hashes()
        self.fingerprint_md5 = c.fingerprint_md5
        self.fingerprint_sha1 = c.fingerprint_sha1
        self.fingerprint_sha256 = c.fingerprint_sha256

        c.pubkey = self.pk_instance

        c.valid_from = self.vdata_cert['notBefore']
        c.expires = self.vdata_cert['notAfter']

        c.user_agent_string = self.user_agent_string
        c.save()


    #####################################################
    #####################################################
    # Public Methods
    #####################################################
    #####################################################

    def create_cert(self):
        """
        sequential calls to all methods
        that, in that order, lead to a valid
        certificate creation
        """
        ######################
        #CERT CREATION PROCESS
        ######################
        self._set_cert_subject()
        self._set_common_name()
        self._set_pubkey()
        self._set_fake_serial()
        self._set_validity()
        self._set_issuer()
        self._set_subject_alt_name()
        self._set_version()

        #XXX can we cache this? better to create it every time???
        #it will add overhead to every request! :(
        #but where do we store the pkey safely?
        #The funny thing is that we only need the signature
        #so the browser's cert store does not crash...

        if not self.skip_sign:
            if not hasattr(self, 'issuerKey'):
                issuerKey = self._create_key_pair(
                        crypto.TYPE_RSA, 1024)
            else:
                issuerKey = self.issuerKey
            self._sign(issuerKey)

        #saving instances
        #XXX transaction block?
        self._save_pubkey_instance()
        self._set_cert_dump()
        self._save_cert_instance()

    def get_cert_dump(self):
        """
        returns a string containing the
        dump of the certificate.
        """
        return self.cert_dump

    def get_b64_cert_dump(self):
        """
        returns a string containing the
        base64 representation of the certificate
        dump
        """
        return base64.b64encode(self.cert_dump)

    def get_sha1_fingerprint(self):
        return self.fingerprint_sha1



###############################################
###############################################
# SOME OTHER OLD FUNCTIONS FROM our legacy code
# (coming from xmppwebid)
###############################################
# To be cleaned and reused
# (pkcs12, i'm looking at you)

def get_serial_from_file(serial_path='/tmp/webid_cert_serial.txt'):
    """Get serial number from file

    :param serial_path: serial file path
    :type serial_path: string
    :return: serial number
    :rtype: int

    :TODO: XXX this serial number is a quick hack.
    this should be tracked in a model table. XXX

    """
    try:
        serial_file = open(serial_path, "r")
        data = serial_file.read()
        serial_file.close()
        if data: number = int(data)
        number += 1
    except:
        number = 1
    serial_file = open(serial_path, "w")
    serial_file.write(str(number))
    serial_file.close()
    return number

def gen_keypair(bits=1024):
    """Create RSA key pair
    Equivalent to: openssl genrsa -des3 -out client.key 1024

    :param bits: key bits length
    :type bits: int
    :return: key
    :rtype: EVP.PKey

    """
    pkey = EVP.PKey()
    rsa = RSA.gen_key(bits, 65537)
    pkey.assign_rsa(rsa)
    #print "Generated private RSA key"
    # Print the new key as a PEM-encoded (but unencrypted) string
    logging.debug(rsa.as_pem(cipher=None))
    return pkey


def gen_csr(pkey):
    """Create an x509 CSR (Certificate Signing Request)
    Equivalent to: openssl req -new -key client.key -out client.csr

    :param pkey: key
    :type pkey: EVP.PKey
    :return: x509 request
    :rtype: X509.Request

    """

    csr = X509.Request()
    csr.set_pubkey(pkey)
    return csr


def gen_cert_from_csr(csr, serial_number=0, years=1):
    """Create an x509 certificate from CSR with default values

    :param csr: x509 certificate request
    :type csr: X509.Request
    :param serial_number: certificate serial number
    :type serial_number: string
    :param years: number of years the certificate is going to be valid
    :type years: int
    :return: x509 certificate
    :rtype: X509.X509

    """
    cert = X509.X509()
    pkey = csr.get_pubkey()
    cert.set_pubkey(pkey)
    # the cert subject is the same as csr subject
    # get subject from request
    x509_name = csr.get_subject()
    cert.set_subject(x509_name) # the same if a csr subject was created
    # set version
    #:TODO: check that changing jabberd version here can remain 3
    cert.set_version(2)
    #:TODO: set a real serial number
    set_serial(cert, serial_number)
    set_valtime(cert, years)
    return cert

def set_cert_httpwebid(cert, webid):
    #XXX REFACTOR with above!!!

    """Set the SubjectAltName, Issuer and Subject

    :param cert: x509 certificate
    :type cert: X509.X509
    :param webid: FOAF WebId
    :type webid: string
    :return: x509 certificate
    :rtype: X509.X509

    """
    # optional
    # the issuer is going to be the same as subject
    x509_name = X509.X509_Name()
    x509_name.O = O
    x509_name.OU = OU
    x509_name.CN = CN
    cert.set_issuer(x509_name)

    # set subjectAltName extension
    ext = X509.new_extension('subjectAltName',
          'URI:%s' %(webid))
    ext.set_critical(1)
    cert.add_ext(ext)
    return cert

def sign_cert(pkey, cert):
    """Sign the cert

    :param pkey: key
    :type pkey: EVP.PKey
    :param cert:  x509 certificate
    :type cert: X509.X509
    :return: x509 certificate, key
    :rtype: X509.X509, EVP.PKey

    """
    cert.sign(pkey, 'sha1')
    return cert, pkey


def gen_httpwebid_selfsigned_cert(webid, serial_number=0, years=1, nick=None):
    """Create an x509 self-signed certificate

    Equivalent to: openssl x509 -req -days 365 -in client.csr
    -signkey client.key -out client.crt

    :param webid: FOAF WebId
    :type webid: string
    :param serial_number: certificate serial number
    :type serial_number: string
    :param years: number of years the certificate is going to be valid
    :type years: int
    :return: x509 self-signed certificate, key
    :rtype: tuple (X509.X509, EVP.PKey)

    """

    pkey = gen_keypair()
    csr = gen_csr(pkey)

    #XXX this changes (refactor)
    csr = set_csr_httpwebid(csr, webid, nick=nick)
    csr, pkey = sign_csr(pkey, csr)
    #XXX

    cert = gen_cert_from_csr(csr, serial_number, years)
    cert = set_cert_httpwebid(cert, webid)
    cert, pkey = sign_cert(pkey, cert)

    # Print the new certificate as a PEM-encoded string
    #print "Generated new self-signed client certificate"
    logging.debug(cert.as_pem())

    return cert, pkey



def gen_httpwebid_selfsigned_cert_pemfile(webid,
        cert_path='/tmp/webid_cert.pem',
        key_path='/tmp/webid_key.key',
        serial_path='/tmp/webid_cert_serial.txt', years=1, nick=None):
    """Create an x509 self-signed certificate and save it as PEM file

    :param serial_path: serial file path
    :type serial_path: string
    :param webid: FOAF WebId
    :type webid: string
    :param years: number of years the certificate is going to be valid
    :type years: int
    :param cert_path: certificate path
    :param key_path: key path
    :type cert_path: string
    :type key_path: string
    :return: x509 certificate path, key path
    :rtype: tuple (string, string)

    """
    #XXX get serial from db!!!
    serial_number = get_serial_from_file(serial_path)
    cert, pkey = gen_httpwebid_selfsigned_cert(webid,
            serial_number=serial_number,
            years=years, nick=nick)
    save_pkey_cert_to_pemfile(cert, pkey, cert_path, key_path)
    return cert_path, key_path



##########################################
# PKCS12 SHIT
# ANOTHER SPOT WHERE WE NEED PYOPENSSL !!!
# OR A PROPER PKCS12 WRAPPER!!!
# (I keep thinking that pkcs12 as a fallback
# is a good thing... f.i., for some mobile
# users...
##########################################

def save_pkcs12cert_to_pkcs12file(p12, p12cert_path='/tmp/webid_cert.p12'):
    """Save PKCS12 certificate to file

    :param p12: PKCS12 certificate
    :type p12: OpenSSL.crypto.PKCS12
    :param p12cert_path: PKCS12 certificate path
    :type p12cert_path: string
    :return: PKCS12 certificate path
    :rtype: string

    """
    p12cert = open(p12cert_path,"w")
    p12cert.write(p12.export())
    p12cert.close()
    return p12cert_path


def pemfile_2_pkcs12file(cert_path='/tmp/webid_cert.pem',
        key_path='/tmp/webid_key.key',
        p12cert_path='/tmp/webid_cert.p12'):
    """Create a PKCS12 certificate and save it from x509 certificate and
    key files as PEM

    :param cert_path: certificate path
    :param key_path: key path
    :param p12cert_path: key path
    :type cert_path: string
    :type key_path: string
    :type p12cert_path: string
    :return: PKCS12 certificate path
    :rtype: string

    """
# need the OpenSSL type
#    pkey = get_pkey_from_pkeypemfile(key_path)
#    cert = get_cert_from_certpemfile(cert_path)
    pkey = crypto.load_privatekey(OpenSSL.SSL.FILETYPE_PEM,
                                          open(key_path).read())
    cert = crypto.load_certificate(OpenSSL.SSL.FILETYPE_PEM,
                                           open(cert_path).read())
    p12 = pkey_cert_2_pkcs12cert(cert, pkey)
    p12cert_path = save_pkcs12cert_to_pkcs12file(p12, p12cert_path)
    return p12cert_path




from django.contrib import admin
from .models import CertConfig, PubKey, Cert, WebIDUser

class CertConfigAdmin(admin.ModelAdmin):
    """
    Admin class for the CertConfig model.
    """
    #XXX Add a Site behavior section.
    fieldsets = (
            ('App Behavior', {
                'fields': (
                    'hide_keygen_form',)}),
            ('Default Cert Subject', {
                'fields': (
                    'country_name',
                    'state',
                    'locality',
                    'organization',
                    'organizational_unit',
                    'common_name_field')}),
            ('Validity', {
                'classes': ('collapse',),
                'fields': (
                    'valid_for_days',
                    'valid_from_days')}),
                )

    def has_add_permission(self, request):
        """
        CertConfig behaves as a singleton.
        We populate it from the config when first syncing.
        """
        return False

    def has_delete_permission(self, request, obj=None):
        """
        CertConfig behaves as a singleton.
        We populate it from the config when first syncing.
        """
        #XXX FIXME should remove also the delete-action!!!
        return False

class CertAdmin(admin.ModelAdmin):
    """
    Admin class for Cert model.
    """
    readonly_fields = ( 'user_agent_string',
                        'fingerprint_sha256',
                        'fingerprint_sha1',
                        'fingerprint_md5')

class PubKeyAdmin(admin.ModelAdmin):
    """
    Admin class for PubKey model.
    """
    readonly_fields = ( 'mod',
                        'exp',
                        'bits',
                        'pkey_algorithm')

    #XXX FIXME get a better display
    #for the mod. beautify for the admin display??

admin.site.register(CertConfig, CertConfigAdmin)

#FIXME XXX Pubkey/Cert should be using Inlines.
admin.site.register(PubKey, PubKeyAdmin)
admin.site.register(Cert, CertAdmin)

#FIXME XXX WebIDUser could also use inlines to pubkeys.
admin.site.register(WebIDUser)



VERSION = (0, 1, 2)


def get_version():
    version = '%s.%s' % (VERSION[0], VERSION[1])
    if VERSION[2]:
        version = '%s.%s' % (version, VERSION[2])
#    if VERSION[3] != "final":
#        version = '%s%s%s' % (version, VERSION[3], VERSION[4])
    return version

__version__ = get_version()



from django.db.models.signals import post_syncdb
from django_webid.provider import models as webidprovider_app

from .certconfig_defaults import set_defaultsettings

post_syncdb.connect(set_defaultsettings, sender=webidprovider_app)



#http://djangosnippets.org/snippets/1518/
import urlparse
from django.template import Library
from django.template.defaulttags import URLNode, url
from django.contrib.sites.models import Site

register = Library()

class AbsoluteURLNode(URLNode):
    def render(self, context):
        path = super(AbsoluteURLNode, self).render(context)
        domain = "http://%s" % Site.objects.get_current().domain
        return urlparse.urljoin(domain, path)

def absurl(parser, token, node_cls=AbsoluteURLNode):
    """Just like {% url %} but ads the domain of the current site."""
    node_instance = url(parser, token)
    return node_cls(view_name=node_instance.view_name,
        args=node_instance.args,
        kwargs=node_instance.kwargs,
        asvar=node_instance.asvar)
absurl = register.tag(absurl)



from django import template

register = template.Library()

#XXX this can be made smarter, but...
DEPRECATED_PUBKEY_SNIPPET = u"""
<div about="#cert" typeof="rsa:RSAPublicKey">
  <div href="%(uri)s" rel="cert:identity"></div>
  <div property="rsa:modulus" datatype="cert:hex" content="%(mod)s">%(pretty_mod)s</div>
  <div property="rsa:public_exponent" datatype="cert:int" content="%(exp)s">%(exp)s</div>
  <div about="%(uri)s"></div>
</div>
"""

#XXX :( I was liking the %(pretty_mod)s...

PUBKEY_SNIPPET_RDFA = u"""
<div rel="cert:key">
    <p>One of my Public Keys... made on...</p>
    <div typeof="cert:RSAPublicKey">
      <dl>
      <dt>Modulus (hexadecimal)</dt>
      <dd style="word-wrap: break-word; white-space: pre-wrap;"
         property="cert:modulus" datatype="xsd:hexBinary">%(mod)s</dd>
      <dt>Exponent (decimal)</dt>
      <dd property="cert:exponent" datatype="xsd:integer">%(exp)s</dd>
      </dl>
    </div>
</div>
"""
PUBKEY_SNIPPET_RDFXML = u"""
    <cert:key>
      <cert:RSAPublicKey>
        <rdfs:label>made on [...] on my laptop</rdfs:label>
        <cert:modulus rdf:datatype="http://www.w3.org/2001/XMLSchema#hexBinary">
        %(mod)s
        </cert:modulus>
        <cert:exponent rdf:datatype="http://www.w3.org/2001/XMLSchema#integer">%(exp)s</cert:exponent>
      </cert:RSAPublicKey>
    </cert:key>"""

PUBKEY_SNIPPET_TURTLE = u"""
  :key [ a :RSAPublicKey;
    rdfs:label "key made on [...] on my laptop";
    :modulus
   "%(mod)s"^^xsd:hexBinary;
    :exponent %(exp)s ;
  ] .
"""

PADDING_CHR = u'\u271c'
PADDING_SPACES =  u' %s ' % PADDING_CHR
TRAILING_CHR = u'\u2665'

def prettyfy(hexstr):
    return PADDING_SPACES.join([
        hexstr[x:x+2].upper()
        for x in xrange(0,len(hexstr),2)
        ]) + ' %s' % TRAILING_CHR

def do_pubkey_rdf(parser, token):
    try:
        tag_name, _format, user_var = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("%r tag requires a single argument" % token.contents.split()[0])
    if not (_format[0] == _format[-1] and _format[0] in ('"', "'")):
        raise template.TemplateSyntaxError("%r tag's argument should be in quotes" % tag_name)
    return_format = _format[1:-1]
    if not return_format in ('rdfa', 'rdfxml', 'turtle'):
        raise template.TemplateSyntaxError("%r tag format argument should be one of the following: 'rdfa', 'rdfxml'" % tag_name)
    return PubKeyRDFNode(return_format, user_var)


class PubKeyRDFNode(template.Node):
    def __init__(self, _format, user_var):
        self._format = _format
        self.user_var = template.Variable(user_var)
    def render(self, context):
        try:
            #XXX REFACTOR
            uu = self.user_var.resolve(context)
            if self._format == "rdfa":
                r = ""
                for pk in uu.keys:
                    r = r + PUBKEY_SNIPPET_RDFA % {'mod':pk.mod,
                        'pretty_mod': prettyfy(pk.mod),
                        'exp':pk.exp,
                        'uri': uu.absolute_webid_uri,
                        }
                return r
            if self._format == "turtle":
                r = ""
                for pk in uu.keys:
                    r = r + PUBKEY_SNIPPET_TURTLE % {'mod':pk.mod,
                        'exp':pk.exp,
                        }
                return r
            if self._format == "rdfxml":
                r = ""
                for pk in uu.keys:
                    r = r + PUBKEY_SNIPPET_RDFXML % {'mod':pk.mod,
                        'exp':pk.exp,
                        }
                return r
            raise template.TemplateSyntaxError("format should be one of the following: 'rdfa', 'rdfxml'")
        except template.VariableDoesNotExist:
            return ''

register.tag('pubkey_rdf', do_pubkey_rdf)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages
import os

execfile('./src/django_webid/provider/__init__.py')
VERSION = __version__

setup_root = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(setup_root, "src"))

long_description = "A reusable django app that turns any django-powered \
site into a WebID provider"""

packages = find_packages('src')
print 'packages', packages
setup(
    name='django-webid-provider',
    package_dir={'': 'src'},
    packages=packages,
    namespace_packages=['django_webid'],
    include_package_data=True,
    exclude_package_data={
        'requirements': ['%s/*.tar.gz' % VERSION],
    },
    version=VERSION,
    description="A reusable django app that turns any django-powered site into \
a WebID provider""",
    long_description=long_description,
    author='Ben Carrillo',
    author_email='bennomadic@gmail.com',
    download_url='https://github.com/bennomadic/django-webid-provider/tarball/v0.1',
    url="https://github.com/bennomadic/django-webid-provider",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: JavaScript',
        'Framework :: Django',
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Security :: Cryptography",
    ],
    keywords='foaf, ssl, webid, x509, certificate, client certificate,\
authentication, authorization,django',
    zip_safe=False,
)



#!/usr/bin/python
# vim: set expandtab tabstop=4 shiftwidth=4:
# -*- coding: utf-8 -*-

#
# Copyright (C) 2011 bennomadic at gmail dot com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
"""
    views
    ~~~~~~~~

    :authors:       Ben Carrillo, Julia Anaya
    :organization:  rhizomatik labs
    :copyright:     Cooperative Quinode
    :license:       GNU GPL version 3 or any later version
                    (details at http://www.gnu.org)
    :contact:       bennomadic at gmail dot com
    :dependencies:  python (>= version 2.6)
    :change log:
    :TODO:
"""
import hashlib
import logging
import os
import random
import re

#from datetime import datetime

from django import template

from django.contrib.auth import logout
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect,\
        get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_protect
from django.views.generic import list_detail
from django.conf import settings
from django.db import models


#XXX this import should be handled on certs.utils now
from OpenSSL import crypto

#XXX FIXME fix relative imports
#from .models import PubKey,
from .models import WebIDUser, CertConfig, Cert
#from .certs.utils import create_cert # deprecated
from .certs.utils import CertCreator


#XXX PKCS12 SHIT
#... that we could shut down by the moment...
from .certs.utils import gen_httpwebid_selfsigned_cert_pemfile, pemfile_2_pkcs12file
from .forms import WebIdIdentityForm


from django.contrib.auth import authenticate, login as auth_login
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required

from django.db.models.signals import post_save
from django.dispatch import receiver




#############################################################
# VIEWS BEGIN
#############################################################


################
# CREATE USER
################

#XXX we should hook django register app here, instead of this

def create_user(request):
    """
    creates user and redirects to the page
    for installing a cert.
    """
    #XXX should check it's anon_login,
    #or redirect to logout in other case.

    #This view SHOULD NOT be available if this config
    #is not allowed in settings. We can do that by
    #manually removing from urls.
    #We could also put here a decorator that checks
    #the settings and allows/denies access to view.

    #XXX TODO
    #on this view (or another different??) we should
    #introduce an **email verification** step.

    errors = False
    messages = []

    if request.method == "POST":
        form = UserCreationForm(request.POST)
        if form.is_valid():
            if not errors:
                u = form.save()

                if settings.DEBUG:
                    logging.debug('created user... %s' % u)
                    logging.debug('active? %s' % u.is_active)

                u.backend = 'django.contrib.auth.backends.ModelBackend'
                auth_login(request, u)

                return redirect('webidprovider-add_cert')


            return render_to_response('django_webid/provider/create_user.html', {
                'form': form,
                "MEDIA_URL": settings.MEDIA_URL,
                "STATIC_URL": settings.STATIC_URL,
                'messages': messages,
                },
                context_instance=RequestContext(request))
    else:
        form = UserCreationForm()

    return render_to_response('django_webid/provider/create_user.html',
     {
        'form': form,
        "MEDIA_URL": settings.MEDIA_URL,
        "STATIC_URL": settings.STATIC_URL,
        'messages': messages,
     }, context_instance=RequestContext(request))


####################
# Log out
###################

def logout_view(request):
    logout(request)
    return redirect('/')

###########################################
# CERTS VIEWS                             #
###########################################

#####################
# CERT:LIST
#####################
@login_required
def cert_list_by_user(request):
    try:
        certs = Cert.objects.filter(pubkey__user=request.user)
    except Cert.DoesNotExist:
        raise Http404

    # Use the object_list view for the heavy lifting.
    return list_detail.object_list(
        request,
        queryset = certs,
        template_name = "django_webid/provider/cert_list.html",
        #XXX not working with template object name??
        #template_object_name = "certs",
    )

#####################
# CERT:DETAIL
#####################
@login_required
def cert_detail(request, cert_id):
    # Look up the Cert (and raise a 404 if she's not found)
    cert = get_object_or_404(Cert, pk=cert_id)
    if cert.pubkey.user != request.user:
        raise Http404

    # Show the detail page
    return list_detail.object_detail(
        request,
        queryset = Cert.objects.all(),
        object_id = cert_id,
        template_name = "django_webid/provider/cert_detail.html"
    )

#####################
# CERT:REVOKE
#####################
def revoke_cert(cert):
    cert.pubkey.is_active = False
    cert.pubkey.save()

@login_required
def cert_revoke(request, cert_id):
    # Look up the Cert (and raise a 404 if not found)
    cert = get_object_or_404(Cert, pk=cert_id)
    if cert.pubkey.user != request.user:
        raise Http404

    if request.method == "POST":
        hidden  = request.POST.get('post', None)
        if hidden and hidden == "yes":
            revoke_cert(cert)
            #XXX we should use messages here.
            return render_to_response('django_webid/provider/cert_revoked.html',
                {
                    "MEDIA_URL": settings.MEDIA_URL,
                    "STATIC_URL": settings.STATIC_URL,
                })
    if request.method == "GET":
        # Show the detail page
        # with a confirmation message.
        return list_detail.object_detail(
            request,
            queryset = Cert.objects.all(),
            object_id = cert_id,
            template_name = "django_webid/provider/cert_revoke.html"
        )

#####################
# CERT:ADD
#####################
@login_required
#XXX doubt... will this decorator work also for webid-auth?
# not logged in, it should check if anon register is allowed
#and redirect there...
#XXX but now it's redirecting to the accounts url that is not
#being loaded...
def add_cert_to_user(request):
    messages = []
    if request.method == 'POST':
        if request.POST.has_key('pubkey'):
            user = request.user
            spkac_str = str(request.POST['pubkey'])

            return_iframe = request.REQUEST.get('iframe', None)
            if return_iframe and return_iframe == "true":
                do_iframe = True
            else:
                do_iframe = False

            return_multipart = request.REQUEST.get('multipart', None)
            if return_multipart and return_multipart == "true":
                do_multipart = True
            else:
                do_multipart = False

            skip_issuer_signing = request.REQUEST.get('skipsign', False)
            if skip_issuer_signing and skip_issuer_signing=="true":
                skip_sign = True
            else:
                skip_sign = False

            #XXX should catch exceptions here (bad pubkey???)
            #XXX or tampered challenge :)

            kwargs = {}
            kwargs['user_agent_string'] = request.META['HTTP_USER_AGENT']
            kwargs['skip_sign'] = skip_sign
            #XXX TODO get kwargs from the post advanced fields
            c = CertCreator(spkac_str, user, **kwargs)
            c.create_cert()


            if do_iframe:
                # We create an iframe with b64encoded cert as src.
                # works on FF, opera. Not working in chrome
                #Iframe use like this:
                #http://blog.magicaltux.net/2009/02/09/using-ssl-keys-for-client-authentification/

                certdump = c.get_b64_cert_dump()
                sha1fingerprint = c.get_sha1_fingerprint()


                return render_to_response(
                        'django_webid/provider/webid_b64_cert.html',
                        {
                            "MEDIA_URL": settings.MEDIA_URL,
                            "STATIC_URL": settings.STATIC_URL,
                            "b64cert": certdump,
                            "sha1fingerprint": sha1fingerprint,
                            "user": request.user,
                            }, context_instance=RequestContext(request))

            if do_multipart:
                # As a workaround for iframes, experimenting with multipart
                # black magic
                # XXX Here Be Dragons
                # chrome also misbehaves with this :(
                certdump = c.get_cert_dump()
                #r = HttpResponse(mimetype="application/x-x509-user-cert")
                BOUND = "BOUND"
                r = HttpResponse(
                        mimetype="multipart/x-mixed-replace;boundary=%s" % BOUND)

                N = "\r\n"
                NN = "\r\n\n"

                r.write(BOUND + N + 'Content-Type: application/x-x509-user-cert' + NN)
                r.write(certdump)
                r.write(N + "--" + BOUND + N + 'Content-Type: text/html' + NN)
                r.write("<html><body><h1>hello world! did you see my cert???</h1></body></html>")
                r.write(N + "--" + BOUND + "--")
                return r

            if not do_iframe:
                # We deliver the cert as the only response
                certdump = c.get_cert_dump()
                r = HttpResponse(mimetype="application/x-x509-user-cert")
                r.write(certdump)
                return r



    app_conf = CertConfig.objects.single()
    r = random.getrandbits(100)
    challenge = hashlib.sha1(str(r)).hexdigest()

    return render_to_response('django_webid/provider/webid_add_to_user.html',
        {
        "HIDE_KEYGEN_FORM": app_conf.hide_keygen_form,
        "MEDIA_URL": settings.MEDIA_URL,
        "ADMIN_MEDIA_PREFIX": settings.ADMIN_MEDIA_PREFIX,
        "STATIC_URL": settings.STATIC_URL,
        "challenge": challenge,
        'messages': messages,
        'user':request.user},
        context_instance=RequestContext(request))


###########################################
# WEBID VIEWS                             #
###########################################
# Deprecated by the content-negotiated view
# in webiduri

def render_webid(request, username=None):
    uu = get_object_or_404(WebIDUser,
            username=username)
    return render_to_response('django_webid/provider/foaf/webid_rdfa.html',
             {
             "webiduser": uu,
             "MEDIA_URL": settings.MEDIA_URL,
             "STATIC_URL": settings.STATIC_URL,
             })
             #, context_instance=RequestContext(request))






###################################################
###########################################
# OLD VIEWS THAT WE CAN KEEP BUT NEED TO BE
# REFACTORED TO USE NEW FUNCTIONS
###########################################
###################################################


def webid_identity_keygen(request):
    """
    Create only http-WebID certificate
    using keygen in the browser
    """
    #XXX FIXME
    # REFACTOR using above functions... :/
    #XXX TODO:
    #Detect if using IExplorer (support needed?)
    #and leave the .p12 as a fallback mechanism.
    #they say it's a good fallback for things like iphone
    #sending the pkcs12 (w/ pass) by email.

    #XXX TODO: try also javascript CSR creation... which
    #format does it produce??

    errors = False
    messages = []

    if request.method == 'POST':
        form = WebIdIdentityForm(request.POST)
        if form.is_valid():
            if not errors:
                webid = str(form.cleaned_data['webid'])
                nick = str(form.cleaned_data['nick'])

                #XXX REFACTOR ##############################
                pubkey = re.sub('\s', '', str(request.POST['pubkey']))
                if settings.DEBUG:
                    logging.debug('PUBKEY=%s' % pubkey)
                    #print('PUBKEY=%s' % pubkey)
                spki = crypto.NetscapeSPKI(pubkey)
                cert = crypto.X509()
                cert.get_subject().C = "US"
                cert.get_subject().ST = "Minnesota"
                cert.get_subject().L = "Minnetonka"
                cert.get_subject().O = "my company"
                cert.get_subject().OU = "WebID"
                cert.get_subject().CN = nick
                cert.set_serial_number(001)
                cert.gmtime_adj_notBefore(0)
                cert.gmtime_adj_notAfter(10*365*24*60*60)
                cert.set_issuer(cert.get_subject())
                URI_STR = 'URI:%s' % (webid)
                ext = crypto.X509Extension('subjectAltName', 1,
                    URI_STR)
                cert.add_extensions([ext])
                cert.set_version(2) # version 3 (decimal)
                #we get the pubkey from the spkac
                cert.set_pubkey(spki.get_pubkey())
                res = crypto.dump_certificate(crypto.FILETYPE_ASN1, cert)
                r = HttpResponse(mimetype="application/x-x509-user-cert")
                r.write(res)
                return r
                #XXX REFACTOR ##############################

            return render_to_response('django_webid/provider/webid_identity_keygen.html', {
                'form': form,
                "MEDIA_URL": settings.MEDIA_URL,
                "STATIC_URL": settings.STATIC_URL,
                'messages': messages,
            }, context_instance=RequestContext(request))
    else:
        form = WebIdIdentityForm() # An unbound form

    return render_to_response('django_webid/provider/webid_identity_keygen.html', {
        #XXX TODO: add randomchars challenge...
        'form': form,
        "MEDIA_URL": settings.MEDIA_URL,
        "STATIC_URL": settings.STATIC_URL,
        'messages': messages,
    }, context_instance=RequestContext(request))



def webid_identity(request):
    """
    Create only http-WebID certificate
    """

    #XXX TODO
    #this is very similar to above
    #function, but creates the cert as a pkcs12.
    #XXX TODO CHANGE VIEW NAME!!! --> TO PKCS12_IDENTITY
    #OR SOMETHING LIKE THAT...

    #REFACTOR!! ################################

    #Detect if using IExplorer (support needed?)
    #and leave the .p12 as a fallback mechanism.

    errors = False
    messages = []

    if request.method == 'POST':
        form = WebIdIdentityForm(request.POST)
        if form.is_valid():
            if not errors:
                webid = str(form.cleaned_data['webid'])
                nick = str(form.cleaned_data['nick'])
                try:
                    gen_httpwebid_selfsigned_cert_pemfile(webid, nick=nick)
                    #XXX THIS IS THE SECOND MIX USE OF OPENSSL/M2CRYPTO
                    path = pemfile_2_pkcs12file()
                    #print "PKC12 path: " + path
                except Exception, e:
                    message = "Error trying to generate client certificate: " + str(e)
                    #print message
                    messages.append(message)
                    # can not continue
                else:
                    fp = open(path)
                    content = fp.read()
                    fp.close()
                    length = os.path.getsize(path)
                    r = HttpResponse(mimetype="application/x-x509-user-cert")
                    #handle = webid.split('/')[-1] # XXX ugly, but has to be something!
                    r['Content-Disposition'] = 'attachment; filename=%s%s' % (nick, "_cert.p12")
                    r["Content-Length"] = length
                    r["Accept-Ranges"] ="bytes"
                    r.write(content)
                    messages.append("created")
                    return r
#            finally:
            # if exception or not is happend and no return is already executed
            return render_to_response('django_webid/provider/webid_identity.html', {
                'form': form,
                "MEDIA_URL": settings.MEDIA_URL,
                "STATIC_URL": settings.STATIC_URL,
                'messages': messages,
            }, context_instance=RequestContext(request))
    else:
        form = WebIdIdentityForm()

    return render_to_response('django_webid/provider/webid_identity.html', {
        'form': form,
        "MEDIA_URL": settings.MEDIA_URL,
        "STATIC_URL": settings.STATIC_URL,
        'messages': messages,
    }, context_instance=RequestContext(request))





###########################################
#            SIGNALS                      #
###########################################


@receiver(post_save, sender=User)
def init_blank_profile_for_new_user(sender, **kwargs):
    """
    initializes a blank profile for webiduser
    in the moment of its creation.
    """
    if kwargs.get('created', None):
        user = kwargs.get('instance')
        webiduser = WebIDUser.objects.get(id=user.id)

        profile_model = settings.AUTH_PROFILE_MODULE
        app_split = profile_model.split('.')
        if len(app_split) == 2:
            app_label, mod_label = app_split
        else:
            app_label, mod_label = app_split[-2:]
        pklss = models.get_model(app_label, mod_label)

        pklss.objects.create(user=webiduser)



import datetime
from OpenSSL import crypto

from django.utils import unittest
from django.contrib.auth.models import User

from django_webid.provider.certs.utils import CertCreator
from django_webid.provider.models import Cert

TEST_ICEWEASEL_SPKAC="MIICaDCCAVAwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCbPAp1rVLNOXTK3Lie0+0pKiJSOvMhvtSm2WBZQ4wvqtjpm1Ru45LR1d3Cq2JvJgSW2Ou9DqeKduhk8vwaxyALCV5wpKyVkJy6nA1e8IBL+NCg51pN4pIHW3O+0GRoMqgEnJn42JLocmDnUort4gHQZX8Ti9qYp11+LaIkhriWD9lDOq3hRgPAopdYEt5PbNkA//1d5EyhPnGac3MLirUbzCjLeOtaI1FT0+b29SwphbaaO1u7DHeJY52/duJr2/N9dJKdu+zDemvCgnz0QvmJRepbBDhjb0YprNeSLykNjVxZgQGBcxY8RZrNGajtwYQKZWb9LfLQVKIM1Tw3qBlJAgMBAAEWKGMxM2MxYjQ2ZjY5YjE0OWNlZDAxMmEyZDA4MzQxODg1MTMyODYwODQwDQYJKoZIhvcNAQEEBQADggEBAEvjEFNzPW9dpKzDhInyKPO1XH0p9cS9L5EopuXe2ZysB8C975BTUyfnLRLlB9aMFie6LZ5xbcVRAYiXtar57IGAslUP65WQmvUwnzYCi48Z29P0QU7VGkncoI5JTy4pOGkFHOnQUw+1NR3FL9owS91up7QiZ1RIsSsfyEZvDIJ/1iKZC5V7pK/oDy8bYBissImyfpn5rAG6cUrwZW8d2axh62F1kUVnMyftGsQy6+pU3DFE/GDK2ZhjWdJlvA+I1oIz7/Tg7sU2PK2mY+b7KuO0NeGVy2rrDdveuR4Ys8yLCm3iQD0iS0y8sZX05ee1a60Tng/qJy9dnQeUAoYVAsg="

TEST_MSIE_PKCS10=''

TEST_ISSUER_PKEY = '-----BEGIN PRIVATE KEY-----\nMIICeQIBADANBgkqhkiG9w0BAQEFAASCAmMwggJfAgEAAoGBAMIScjmDtte3FSXy\nB54nlNij65OK/tgR5UIKHY311sVa+a3P1Tkh8r+kOPzbLcUewRXl5IB74l8t4NhA\n6t2aL1oLpDZpFvXkUsNEdhNbgJ5Wjz/gwAxJ3Hyx0MwJZzbWrt0sjJLSpL0ClZe1\ncQXOu41wfA4VZCd0s18weSziyGHbAgMBAAECgYEAu6n0tHMOgsfydk/1V9T0lQzl\nhzXYOv4LD/ISJC0+cncHizC3Adk3EGPwC2qydrPHiLJWZHvnKZyGmPclZgZrk9ng\nYIZ/ZpT1/DTyUk3LeF+rgsOIkDIgNSijuH9kXNXN8r92eiqXkwHk4LMPZ3wV0OxV\nEO+SISj8ps5GgQuVvwECQQDw3AaPyeqNwaaiIEWzFe35PQnM2OwaCVjE27XDNxMl\n9GxLmLaCUMb17+glLjrbAN2ENhlTjYmaC+n81vd5HwNbAkEAzkWBSYRAuk5Cn2BF\n3flvSVJYyiWeejEUWooXGQjDxy1VEZFlv3hIVnNh8ob2YzyJB4ELwH0mMX9oATPL\nK+rjgQJBAMo4AK5SWUIg9vUhYUNlQwJBs+uvqDKH7GaDDIzUvZfdKdsiYQDyLskn\nXeFxeeqLRHAPN55Fs+SI4i/sj6O6XQcCQQCWM1KDoAQqJdapi0cU7g81SvtQp7gQ\nrjBuBWPwXMuC++WYF1IJ7KJwITDPk6tSc8AscLGIBxmKrYWkanyljDMBAkEAgc/t\ny6BXB9+2q37HfioRe7dV8/UlifNSli3JOU38UKiBuXaZ43cGzDkPqJM01B0FSosO\nUdqV9yLYuit5BUAKEQ==\n-----END PRIVATE KEY-----\n'

class WebIDCertsTestCase(unittest.TestCase):
    #urls = 'django_webid.provider.urls'
    def setUp(self):
        self.user = User.objects.create_user("test",
                'test@example.com', 'testpw')

    def tearDown(self):
        self.user.delete()

    def test_create_from_spkac(self):
        "Check creation and properties of a cert from a Netscape SPKAC"
        kwargs = {}
        kwargs['valid_from_ts'] = datetime.datetime(2012,1,1,12,0,0)
        kwargs['expires_ts'] = datetime.datetime(2012,12,31,12,0)

        pkey = crypto.load_privatekey(crypto.FILETYPE_PEM,
            TEST_ISSUER_PKEY)
        kwargs['issuerKey'] = pkey

        cc = CertCreator(TEST_ICEWEASEL_SPKAC, self.user, **kwargs)
        cc.create_cert()
        cert = cc.cert

        s = cert.get_subject()
        self.assertTrue(isinstance(cert, crypto.X509))
        self.assertEqual(cert.get_serial_number(), 1L)

        #XXX this should take into account the CertOptions settings.
        self.assertEqual(s.get_components(),
            [('C', 'CC'), ('OU', 'The Community of Self Signers'),
            ('L', '/dev/null'), ('O', 'FOAF+SSL'), ('ST', 'Cyberspace'), ('CN',
                'test')])
        extension = cert.get_extension(0)
        self.assertEqual(extension.get_critical(), 1L)
        #XXX this should also get the WebIDUser
        #method for getting the absolute URI
        #IT Needs the defaultsite app for working.
        self.assertEqual(extension.get_data(),  '0"\x86 http://foafgen.net/webid/test#me')
        self.assertEqual(cert.get_notBefore(), '20120101120000Z')
        self.assertEqual(cert.get_notAfter(), '20121231120000Z')

        db_cert = Cert.objects.get(id=1)
        self.assertEqual(db_cert.fingerprint_sha1,
                'b0:db:47:c4:a8:98:cd:23:9e:a7:50:4b:95:25:a9:11:7f:d1:09:a9')
        self.assertEqual(db_cert.fingerprint_md5,
                'ea:fb:ed:0c:86:3f:ae:aa:4c:c7:c8:ba:7f:3d:ef:e4')
        #print crypto.dump_certificate(crypto.FILETYPE_PEM, cert)

        #Check associated PubKey
        db_pkey = db_cert.pubkey
        self.assertEqual(db_pkey.mod, '9b3c0a75ad52cd3974cadcb89ed3ed292a22523af321bed4a6d96059438c2faad8e99b546ee392d1d5ddc2ab626f260496d8ebbd0ea78a76e864f2fc1ac7200b095e70a4ac95909cba9c0d5ef0804bf8d0a0e75a4de292075b73bed0646832a8049c99f8d892e87260e7528aede201d0657f138bda98a75d7e2da22486b8960fd9433aade14603c0a2975812de4f6cd900fffd5de44ca13e719a73730b8ab51bcc28cb78eb5a235153d3e6f6f52c2985b69a3b5bbb0c7789639dbf76e26bdbf37d74929dbbecc37a6bc2827cf442f98945ea5b0438636f4629acd7922f290d8d5c5981018173163c459acd19a8edc1840a6566fd2df2d054a20cd53c37a81949')
        self.assertEqual(db_pkey.exp, 65537)
        self.assertEqual(db_pkey.bits, 2048)
        self.assertEqual(db_pkey.is_active, True)
        self.assertEqual(db_pkey.user, self.user)


    def test_create_cert_from_pkcs10(self):
        "Check creation and properties of a cert from a MSIE PKCS10"
        #TODO
        pass

    #XXX###########################################
    #TODO
    #Check that config has a SITE entry (not example.org)
    #XXX related to the management command that populates the sites table.
    #XXX give a BAD spkac to see it fail :)
